/*********************************************************************
* Copyright (C) 2010 by Dario Ignacio Aguilera                       *
* dario_21_06@hotmail.com                                            *
*                                                                    *
* This program is free software; you can redistribute it and/or      *
* modify it under the terms of the GNU General Public License        *
* as published by the Free Software Foundation; either version 2     *
* of the License, or (at your option) any later version.             *
*                                                                    *
* This program is distributed in the hope that it will be useful,    *
* but WITHOUT ANY WARRANTY; without even the implied warranty of     *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
* GNU General Public License for more details.                       *
*                                                                    *
* You should have received a copy of the GNU General Public License  *
* along with this program; if not, write to the                      *
* Free Software Foundation, Inc.                                     *
* 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.      *
**********************************************************************/

#include <QtGui>

#include "mainwindow.h"
#include "imagewidget.h"
#include "zoomwidget.h"
#include "aboutdialog.h"
#include "fileproperties.h"
#include "configdialog.h"
#include "settings.h"

MainWindow::MainWindow()
{
    settings = new Settings;
    settings->loadSettings();

    //configure the scrollarea
    scrollArea = new QScrollArea;
    scrollArea->setBackgroundRole(QPalette::Mid);
    scrollArea->setAlignment(Qt::AlignCenter);
    scrollArea->setFrameShape(QFrame::NoFrame);
    setCentralWidget(scrollArea);

    //create the image widget and make the conections
    imageWidget = new ImageWidget;
    scrollArea->setWidget(imageWidget);
    connect(imageWidget, SIGNAL(couldOpen(bool)), this, SLOT(couldOpen(bool)));
    connect(imageWidget, SIGNAL(pixmapChanged()), this, SLOT(pixmapChangedSlot()));
    connect(imageWidget, SIGNAL(moveWidget(QPoint, QPoint)), this, SLOT(moveWidget(QPoint, QPoint)));

    //the zoom widget
    zoomWidget = new ZoomWidget;
    connect(imageWidget, SIGNAL(wheelZoom(int)), this, SLOT(wheelZoom(int)));
    connect(imageWidget, SIGNAL(picSizeChanged()), this, SLOT(imageTranformated()));
    connect(zoomWidget, SIGNAL(zoom(double)), imageWidget, SLOT(makeZoom(double)));
    connect(zoomWidget, SIGNAL(zoom(double)), this, SLOT(updateWindowTitle()));

    setNameFilters();
    createActions();
    createMenus();
    createToolBar();
    //createContextMenu();

    archivo = new QFileInfo;
    imageSetted = false;
    pixmapChanged = false;

    //add actions to imageWidget, zoomin, zoomout, separator, etc
    imageWidget->addAction(zoomInAct);
    imageWidget->addAction(zoomOutAct);
    //a separator
    QAction *action1 = new QAction(this);
    action1->setSeparator(true);
    imageWidget->addAction(action1);    
    imageWidget->addAction(normalSizeAct);
    imageWidget->addAction(adjustSizeAct);    
    //another separator
    QAction *action2 = new QAction(this);
    action2->setSeparator(true);
    imageWidget->addAction(action2);    
    imageWidget->addAction(filePropertiesAct);
    imageWidget->setContextMenuPolicy(Qt::ActionsContextMenu);

    loadSettings();

    this->setWindowTitle(tr("QIviewer"));
    this->setWindowIcon(QIcon::fromTheme("qiviewer"));
    this->showMenuBar();
    this->addAction(this->showMenuBarAct);
    this->addAction(this->setTBMovableAct);
    this->setContextMenuPolicy(Qt::ActionsContextMenu);
    
#ifdef WEBP_SUPPORT
    qDebug()<<"webp support enabled";
#else
    qDebug()<<"webp support disabled";
#endif
}

void MainWindow::moveWidget(QPoint d, QPoint e)
{
    //don't ask, i just toke this method code from gwenview (lib/scrolltool) and i don't know how it works
    //of course, i had to make some tweaks
    int width = scrollArea->width();
    int height = scrollArea->height();

    QPoint mousePos = d;

    int deltaX = e.x() - mousePos.x();
    int deltaY = e.y() - mousePos.y();

    if (mousePos.y() <= 4 && scrollArea->verticalScrollBar()->value() < scrollArea->verticalScrollBar()->maximum() - 10) {
            // wrap mouse from top to bottom
            mousePos.setY(height - 5);
    } else if (mousePos.y() >= height - 4 && scrollArea->verticalScrollBar()->value() > 10) {
            // wrap mouse from bottom to top
            mousePos.setY(5);
    }

    if (mousePos.x() <= 4 && scrollArea->horizontalScrollBar()->value() < scrollArea->horizontalScrollBar()->maximum() - 10) {
            // wrap mouse from left to right
            mousePos.setX(width - 5);
    } else if (mousePos.x() >= width - 4 && scrollArea->horizontalScrollBar()->value() > 10) {
            // wrap mouse from right to left
            mousePos.setX(5);
    }

    scrollArea->horizontalScrollBar()->setValue(scrollArea->horizontalScrollBar()->value() + deltaX);
    scrollArea->verticalScrollBar()->setValue(scrollArea->verticalScrollBar()->value() + deltaY);
}
void MainWindow::imageTranformated()
{
    if(!adjustSizeAct->isChecked())
        return;

    zoomWidget->clearSizes();
    zoomWidget->setPicSize(imageWidget->getPictureSize());
    zoomWidget->setMaxSize(centralWidget()->size() - QSize(20,20));
    if(showZoomSlider){
        zoomSlider->setValue(zoomWidget->getZoomValue());
    }
    updateActions();
}

void MainWindow::loadAppCommands(const QStringList d)
{
    //sets the file
    archivo->setFile(d.at(1));

    bool open;
    int i, j;
    i=0; j=nameFilters.size(); open = false;

    //check if the file format is included in the supported formats
    //in case it is, open=true, but open=false
    QString g;
    while(!open && i<j){
        g = nameFilters.at(i);
        g.remove('*');
        g.remove('"');
        if(archivo->absoluteFilePath().indexOf(g) != -1){
            open = true;
            i = j;
        }
        i++;
    }

    //if open=true, the app explore the folder and open the image
    if(open){
        exploreFolder();
        openImage();
    }

    //if open=false, show a message
    else{
        QMessageBox notOpen;
        notOpen.setIcon(QMessageBox::Information);
        notOpen.setText(tr("QIviewer couldn't open the image, what do you want to do?"));
        notOpen.setStandardButtons(QMessageBox::Ok | QMessageBox::Open);
        notOpen.setDefaultButton(QMessageBox::Ok);
        int ret = notOpen.exec();
        switch (ret) {
           case QMessageBox::Ok:
               break;
           case QMessageBox::Open:
               lastDirUsed = archivo->absolutePath();
               this->open();
               break;
           default:
               // should never be reached
               break;
         }
    }
}

void MainWindow::configureProgram()
{
    //create a list with the shortcuts
    /*QStringList b;//sc
    for(int i=0; i<actionsList.size(); i++){
        if(actionsList.at(i)->shortcut().isEmpty()){
            b<<QString("none");
        }
        else{
            b<<actionsList.at(i)->shortcut().toString();
        }
    }*/
    saveSettings();
    ConfigDialog con(this);
    con.setWindowTitle(tr("Configuration"));
    con.setActionsList(allActions,QStringList(),iconsName);
    connect(&con, SIGNAL(settingsSaved()), this, SLOT(loadSettings()));
    con.exec();
}

void MainWindow::resizeEvent(QResizeEvent *)
{
    if(imageSetted){
        zoomWidget->maxSizeChanged(centralWidget()->size() - QSize(20,20));
    }
}

void MainWindow::configureToolBar(QStringList sl, Qt::ToolBarArea a, Qt::ToolButtonStyle s, bool v, bool zl)
{
    //clear tool bar
    mainToolBar->clear();

    //make it visible, or not
    mainToolBar->setVisible(v);

    //add the actions
    for(int i=0; i < sl.size(); i++){
        if(sl.at(i) == tr("separator")){
            mainToolBar->addSeparator();
        }
        for(int j=0; j < actionsList.size(); j++){
            QString name = actionsList.at(j)->text();
            name.remove('&');
            if(name == sl.at(i)){
                mainToolBar->addAction(actionsList.at(j));
            }
        }
    }

    //add the toolbar to the mainWindow
    addToolBar(a, mainToolBar);

    //settning button style
    mainToolBar->setToolButtonStyle(s);

    //zoom slider stuff
    if(zl){
        zoomSlider = new QSlider(Qt::Horizontal);
        zoomSlider->setMaximumSize(QSize(75, 25));
        zoomSlider->setRange(zoomWidget->getMinZoomValue(),
                             zoomWidget->getMaxZoomValue());
        connect(zoomWidget, SIGNAL(sliderSignal(int)), zoomSlider, SLOT(setValue(int)));
        connect(zoomSlider, SIGNAL(valueChanged(int)), zoomWidget, SLOT(sliderSlot(int)));
        mainToolBar->addSeparator();
        mainToolBar->addWidget(zoomSlider);
        zoomSlider->setEnabled(false);
    }
}

void MainWindow::createMenus()
{
    recentFilesMenu = new QMenu(tr("&Recent images"), this);
    recentFilesMenu->setIcon(QIcon::fromTheme("document-open-recent"));

    fileMenu = new QMenu(tr("&File"), this);
    fileMenu->addAction(openAct);
    fileMenu->addAction(openDirAct);
    fileMenu->addMenu(recentFilesMenu);
    fileMenu->addAction(saveAct);
    fileMenu->addSeparator();
    fileMenu->addAction(filePropertiesAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);

    editMenu = new QMenu(tr("&Edit"), this);
    editMenu->addAction(showMenuBarAct);
    editMenu->addAction(setTBMovableAct);
    editMenu->addAction(configAct);

    viewMenu = new QMenu(tr("&View"), this);
    viewMenu->addAction(zoomInAct);
    viewMenu->addAction(zoomOutAct);
    viewMenu->addSeparator();
    viewMenu->addAction(normalSizeAct);
    viewMenu->addAction(adjustSizeAct);

    imageMenu = new QMenu(tr("&Image"), this);
    imageMenu->addAction(rotateLeftAct);
    imageMenu->addAction(rotateRightAct);
    imageMenu->addSeparator();
    imageMenu->addAction(flipVerticallyAct);
    imageMenu->addAction(flipHorizontallyAct);

    helpMenu = new QMenu(tr("&Help"), this);
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);

    goMenu = new QMenu(tr("&Go"), this);
    goMenu->addAction(nextAct);
    goMenu->addAction(prevAct);
    goMenu->addSeparator();
    goMenu->addAction(goFirstAct);
    goMenu->addAction(goLastAct);

    menuBar()->addMenu(fileMenu);
    menuBar()->addMenu(editMenu);
    menuBar()->addMenu(viewMenu);
    menuBar()->addMenu(imageMenu);
    menuBar()->addMenu(goMenu);
    menuBar()->addMenu(helpMenu);
}

void MainWindow::createToolBar()
{
    mainToolBar = new QToolBar(tr("Main toolbar"));
    mainToolBar->setFloatable(false);
}

void MainWindow::updateActions()
{
    //this is a complete mess, but i'm too lazy to clean it up
    zoomInAct->setEnabled(imageSetted);
    zoomOutAct->setEnabled(imageSetted);
    saveAct->setEnabled(imageSetted && imageType != Dynamic);
    normalSizeAct->setEnabled(imageSetted);
    adjustSizeAct->setEnabled(imageSetted);
    rotateLeftAct->setEnabled(imageSetted);
    rotateRightAct->setEnabled(imageSetted);
    flipVerticallyAct->setEnabled(imageSetted);
    flipHorizontallyAct->setEnabled(imageSetted);
    if(showZoomSlider)
        zoomSlider->setEnabled(imageSetted);
    filePropertiesAct->setEnabled(imageSetted);
    int g = files.size();
    nextAct->setEnabled(g > 1);
    prevAct->setEnabled(g > 1);
    goFirstAct->setEnabled(g > 1);
    goLastAct->setEnabled(g > 1);
}

void MainWindow::createActions()
{
  openAct = new QAction(tr("&Open"), this);
  openAct->setIcon(QIcon::fromTheme("document-open"));
  openAct->setToolTip(tr("Open an image"));
  connect(openAct, SIGNAL(triggered()), this, SLOT(open()));
  actionsList<<openAct;
  allActions<<tr("Open");
  iconsName<<"document-open";

  saveAct = new QAction(tr("&Save"), this);
  saveAct->setEnabled(false);
  saveAct->setIcon(QIcon::fromTheme("document-save"));
  connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));
  actionsList<<saveAct;
  allActions<<tr("Save");
  iconsName<<"document-save";

  exitAct = new QAction(tr("&Exit"), this);
  exitAct->setIcon(QIcon::fromTheme("application-exit"));
  connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));
  actionsList<<exitAct;
  allActions<<tr("Exit");
  iconsName<<"application-exit";

  filePropertiesAct = new QAction(tr("Properties"), this);
  filePropertiesAct->setEnabled(false);
  filePropertiesAct->setIcon(QIcon::fromTheme("document-properties"));
  connect(filePropertiesAct, SIGNAL(triggered()), this, SLOT(fileProperties()));
  actionsList<<filePropertiesAct;
  allActions<<tr("Properties");
  iconsName<<"document-properties";

  zoomInAct = new QAction(tr("Zoom In"), this);
  zoomInAct->setEnabled(false);
  zoomInAct->setIcon(QIcon::fromTheme("zoom-in"));
  connect(zoomInAct, SIGNAL(triggered()), this, SLOT(zoomIn()));
  zoomInAct->setShortcut(QKeySequence("+"));
  actionsList<<zoomInAct;
  allActions<<tr("Zoom In");
  iconsName<<"zoom-in";

  zoomOutAct = new QAction(tr("Zoom Out"), this);
  zoomOutAct->setEnabled(false);
  zoomOutAct->setIcon(QIcon::fromTheme("zoom-out"));
  connect(zoomOutAct, SIGNAL(triggered()), this, SLOT(zoomOut()));
  zoomOutAct->setShortcut(QKeySequence("-"));
  actionsList<<zoomOutAct;
  allActions<<tr("Zoom Out");
  iconsName<<"zoom-out";

  normalSizeAct = new QAction(tr("Normal Size"), this);
  normalSizeAct->setEnabled(false);
  normalSizeAct->setIcon(QIcon::fromTheme("zoom-original"));
  connect(normalSizeAct, SIGNAL(triggered()), this, SLOT(normalSize()));
  normalSizeAct->setShortcut(QKeySequence("1"));
  actionsList<<normalSizeAct;
  allActions<<tr("Normal Size");
  iconsName<<"zoom-original";

  adjustSizeAct = new QAction(tr("Best Fit"), this);
  adjustSizeAct->setEnabled(false);
  adjustSizeAct->setCheckable(true);
  adjustSizeAct->setIcon(QIcon::fromTheme("zoom-fit-best"));
  connect(adjustSizeAct, SIGNAL(triggered()), this, SLOT(adjustSizeSlot()));
  actionsList<<adjustSizeAct;
  allActions<<tr("Best Fit");
  iconsName<<"zoom-fit-best";

  rotateRightAct = new QAction(tr("Rotate to right"), this);
  rotateRightAct->setToolTip(tr("Rotate image in the clockwise clock"));
  rotateRightAct->setIcon(QIcon::fromTheme("object-rotate-right"));
  rotateRightAct->setEnabled(false);
  connect(rotateRightAct, SIGNAL(triggered()), this, SLOT(rotateRight()));
  actionsList<<rotateRightAct;
  allActions<<tr("Rotate to right");
  iconsName<<"object-rotate-right";

  rotateLeftAct = new QAction(tr("Rotate to Left"), this);
  rotateLeftAct->setToolTip(tr("Rotate image counter-clockwise to clockwise"));
  rotateLeftAct->setIcon(QIcon::fromTheme("object-rotate-left"));
  rotateLeftAct->setEnabled(false);
  connect(rotateLeftAct, SIGNAL(triggered()), this, SLOT(rotateLeft()));
  actionsList<<rotateLeftAct;
  allActions<<tr("Rotate to Left");
  iconsName<<"object-rotate-left";

  flipVerticallyAct = new QAction(tr("Flip vertically"), this);
  flipVerticallyAct->setToolTip(tr("Turns vertically the image"));
  flipVerticallyAct->setIcon(QIcon::fromTheme("object-flip-vertical"));
  flipVerticallyAct->setEnabled(false);
  connect(flipVerticallyAct, SIGNAL(triggered()), this, SLOT(flipVertically()));
  actionsList<<flipVerticallyAct;
  allActions<<tr("Flip vertically");
  iconsName<<"object-flip-vertical";

  flipHorizontallyAct = new QAction(tr("Flip horizontally"), this);
  flipHorizontallyAct->setToolTip(tr("Reflects the image"));
  flipHorizontallyAct->setEnabled(false);
  flipHorizontallyAct->setIcon(QIcon::fromTheme("object-flip-horizontal"));
  connect(flipHorizontallyAct, SIGNAL(triggered()), this, SLOT(flipHorizontally()));
  actionsList<<flipHorizontallyAct;
  allActions<<tr("Flip horizontally");
  iconsName<<"object-flip-horizontal";

  aboutAct = new QAction(tr("A&bout"), this);
  aboutAct->setIcon(QIcon::fromTheme("help-about"));
  connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
  actionsList<<aboutAct;
  allActions<<tr("About");
  iconsName<<"help-about";

  aboutQtAct = new QAction(tr("About &Qt"), this);
  aboutQtAct->setIcon(QIcon(":/trolltech/qmessagebox/images/qtlogo-64.png"));
  connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
  actionsList<<aboutQtAct;
  allActions<<tr("About Qt");
  iconsName<<"none";

  nextAct = new QAction(tr("Ne&xt"), this);
  nextAct->setStatusTip(tr("Loads next image"));
  nextAct->setIcon(QIcon::fromTheme("go-next"));
  nextAct->setEnabled(false);
  connect(nextAct, SIGNAL(triggered()), this, SLOT(next()));
  actionsList<<nextAct;
  allActions<<tr("Next");
  iconsName<<"go-next";

  goFirstAct = new QAction(tr("Go to the first"), this);
  goFirstAct->setStatusTip(tr("Loads the first image in the folder"));
  goFirstAct->setIcon(QIcon::fromTheme("go-first"));
  goFirstAct->setEnabled(false);
  connect(goFirstAct, SIGNAL(triggered()), this, SLOT(goFirst()));
  actionsList<<goFirstAct;
  allActions<<tr("Go to the first");
  iconsName<<"go-first";

  prevAct = new QAction(tr("Pre&vious"), this);
  prevAct->setStatusTip(tr("Loads previous image"));
  prevAct->setIcon(QIcon::fromTheme("go-previous"));
  prevAct->setEnabled(false);
  connect(prevAct, SIGNAL(triggered()), this, SLOT(previous()));
  actionsList<<prevAct;
  allActions<<tr("Previous");
  iconsName<<"go-previous";

  goLastAct = new QAction(tr("Go to the last"), this);
  goLastAct->setStatusTip(tr("Loads the last image in the folder"));
  goLastAct->setIcon(QIcon::fromTheme("go-last"));
  goLastAct->setEnabled(false);
  connect(goLastAct, SIGNAL(triggered()), this, SLOT(goLast()));
  actionsList<<goLastAct;
  allActions<<tr("Go to the last");
  iconsName<<"go-last";

  openDirAct = new QAction(tr("Open &Folder"), this);
  openDirAct->setIcon(QIcon::fromTheme("folder-open"));
  openDirAct->setStatusTip("Open a folder to explore images inside it");
  connect(openDirAct, SIGNAL(triggered()), this, SLOT(openDir()));
  actionsList<<openDirAct;
  allActions<<tr("Open Folder");
  iconsName<<"folder-open";

  showMenuBarAct = new QAction(tr("Show Menu Bar"), this);
  showMenuBarAct->setCheckable(true);
  showMenuBarAct->setIcon(QIcon::fromTheme("show-menu"));
  connect(showMenuBarAct, SIGNAL(triggered()), this, SLOT(showMenuBar()));
  actionsList<<showMenuBarAct;
  allActions<<tr("Show Menu Bar");
  iconsName<<"show-menu";

  setTBMovableAct = new QAction(tr("Lock toolbar"), this);
  setTBMovableAct->setCheckable(true);
  setTBMovableAct->setStatusTip(tr("Lock/unlock tool bar"));
  connect(setTBMovableAct, SIGNAL(triggered()), this, SLOT(setToolBarMovable()));
  actionsList<<setTBMovableAct;
  allActions<<tr("Look Toolbar");
  iconsName<<"none";

  configAct = new QAction(tr("Configuration"), this);
  configAct->setIcon(QIcon::fromTheme("configure"));
  configAct->setEnabled(true);
  connect(configAct, SIGNAL(triggered()), this, SLOT(configureProgram()));
  actionsList<<configAct;
  allActions<<tr("Configuration");
  iconsName<<"configure";
}

void MainWindow::setToolBarMovable()
{
  mainToolBar->setMovable(!setTBMovableAct->isChecked());
}

void MainWindow::showMenuBar()
{
    if(showMenuBarAct->isChecked()){
        menuBar()->show();
    }
    else{
        menuBar()->hide();
    }
}

void MainWindow::fileProperties()
{
    FileProperties fp;
    fp.setWindowTitle(tr("%1's properties").arg(archivo->baseName()));
    fp.setFile(archivo->absoluteFilePath(), settings->useBinaryStyle(), settings->getFileSizePrecision());
    fp.setPreviewPixmap(imageWidget->getPixmap());
    fp.setPictureSize(imageWidget->getPictureSize());
    fp.exec();
}

void MainWindow::about()
{
  AboutDialog abd(this);
  abd.exec();
}

void MainWindow::flipHorizontally()
{
    imageWidget->transformImage(ImageWidget::horizontally);
}

void MainWindow::flipVertically()
{
  imageWidget->transformImage(ImageWidget::vertically);
}

void MainWindow::rotateRight()
{
    imageWidget->transformImage(ImageWidget::toRight);
}

void MainWindow::rotateLeft()
{
  imageWidget->transformImage(ImageWidget::toLeft);
}

void MainWindow::pushNewFile()
{
    //clean menu and actions
    recentFilesMenu->clear();
    for(int i=0; i<recentFilesAct.size(); i++){
        recentFilesAct.at(i)->~QAction();
    }
    recentFilesAct.clear();

    //remove any previous coincidence
    recentFilesPath.removeAll(archivo->absoluteFilePath());

    //add the file to the top of the list
    recentFilesPath.prepend(archivo->absoluteFilePath());

    //make sure the list doesn't have more files than the allowed
    if(recentFilesPath.size() > maxRecentFiles){
        recentFilesPath.removeLast();
    }

    createRFActs();
}

void MainWindow::openRecentFile()
{
    /*i don't have a clue of how the next stuff wokrs,
     *i just copy/pasted it from the qt examples :-P*/
    QString temp;
    QAction *action = qobject_cast<QAction *>(sender());
    if (action){
        temp = action->data().toString();
    }
    archivo->setFile(temp);

    //check if the file exists
    if(!archivo->exists()){
        QMessageBox::information(this, tr("QIviewer"), tr("The file doesn't exist, may be it's been moved or deleted."),
                                 QMessageBox::Ok);
        recentFilesPath.removeOne(temp);
        createRFActs();
        return;
    }

    exploreFolder();
    openImage();
}

void MainWindow::createRFActs()
{
    while(recentFilesPath.size() > maxRecentFiles){
        recentFilesPath.removeLast();
    }

    //clean menu and actions
    recentFilesMenu->clear();
    for(int i=0; i<recentFilesAct.size(); i++){
        recentFilesAct.at(i)->~QAction();
    }

    recentFilesAct.clear();
    if(recentFilesPath.size() > 0){
        for(int i=0; i < recentFilesPath.size(); i++){
            //creo y agrego un objeto QAction con un icono generico y el nombre del archivo
            recentFilesAct<<new QAction(QIcon::fromTheme("image-x-generic"), QFileInfo(recentFilesPath.at(i)).fileName(), this);

            //agrego la direccion completa a la data del QAction correspondiente
            recentFilesAct.at(i)->setData(recentFilesPath.at(i));

            //connecto la accion con el slot encargado de abrir las imagenes recientes
            connect(recentFilesAct.at(i), SIGNAL(triggered()), this, SLOT(openRecentFile()));

            //agrego la accion al menu
            recentFilesMenu->addAction(recentFilesAct.at(i));
        }
    }
}

void MainWindow::setNameFilters()
{
    nameFilters << "*.png" << "*.pbm"
                << "*.jpg" << "*.jpeg"
                << "*.gif" << "*.tiff"
                << "*.xbm" << "*.JPG"
                << "*.bmp" << "*.ppm"
                << "*.pgm" << "*.mng"
                << "*.xpm"
#ifdef WEBP_SUPPORT
                << "*.webp"
#endif
                << "*.svg" << "*.svgz";
    nameFilters.removeDuplicates();
}

void MainWindow::save()
{
    QStringList g = nameFilters;
    g.removeAll("*.gif");
    g.removeAll("*.mng");
    g.removeAll("*.pbm");
    g.removeAll("*.pgm");
    g.removeAll("*.svg");
    g.removeAll("*.svgz");
#ifdef WEBP_SUPPORT
    g.removeAll("*.webp");
#endif

    QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"),
                                    archivo->absoluteFilePath(),
                                    tr("Image Files (%1)").arg(g.join(" ")));

    if(!(fileName.isEmpty()) && imageWidget->canSave()){
        bool couldsave = imageWidget->getPixmap().save(fileName,0,compressLevel);
        if(couldsave){
            archivo->setFile(fileName);
            this->openImage();
        }
        else{
            QMessageBox::information(this, tr("QIviewer"), tr("I couldn't save the image"), QMessageBox::Ok);
        }
    }
    exploreFolder();
}

void MainWindow::openDir()
{
  QString dirName = QFileDialog::getExistingDirectory(this, tr("Open folder"),
                                                 getLastDir(), QFileDialog::ShowDirsOnly
                                                 | QFileDialog::DontResolveSymlinks);
  exploreFolder(dirName);
}

QString MainWindow::getLastDir() const
{
    QString d;
    switch(pathToUse){
    case 0:
        d = lastDirUsed;
        break;
    case 1:
        d = defaultPath;
        break;
    default:
        d = defaultPath;
        break;
    }
    return d;
}

void MainWindow::open()
{
    setNameFilters();
    if(pixmapChanged){
        int g = QMessageBox::information(this, tr("Image changed"),
                                 tr("I've touched the image so, do you want to "
                                    "save it before exit?"),
                                 QMessageBox::Save, QMessageBox::Discard, QMessageBox::Cancel);
        switch(g){
        case QMessageBox::Save:
            this->save();
            pixmapChanged = false;
            break;
        case QMessageBox::Cancel:
            return;
        }
    }

    QString fileName = QFileDialog::getOpenFileName(this,
                                    tr("Open file", "dialog to open file"),
                                    getLastDir(),
                                    tr("Image Files (%1)").arg(nameFilters.join(" ")));

    if(!(fileName.isEmpty())){
        archivo->setFile(fileName);
        //qDebug()<<"archivo:"<<archivo->absoluteFilePath();
        lastDirUsed = archivo->absolutePath();
        pixmapChanged = false;
        exploreFolder();
        openImage();
    }
}

void MainWindow::exploreFolder(const QString d)
{
    QString path;
    if(!d.isEmpty())
        path = d;

    else
        path = archivo->absolutePath();

    QDir dir(path);
    files = dir.entryList(nameFilters, QDir::Files|QDir::Readable, QDir::Name);
    if(files.size() == 0){
        qDebug()<<tr("No images in folder.");
        return;
    }

    files.sort();
    actualPosition = files.indexOf(archivo->fileName());
    lastDirUsed = path;
    if(!d.isEmpty()){
        actualPosition = 0;
        archivo->setFile(QString("%1/%2").arg(path).arg(files.at(0)));
        openImage();
    }
}

/*
 * if gti = 1, loads next image
 * if gti = -1, loads previous image
 * if gti = 2, loads the last image
 * if gti = -2, loads the first image
 */
bool MainWindow::goToImage(int gti)
{
    int amountOfImages = files.size();
    if(amountOfImages==1){
      return false;
    }
    QString temp;
    if(gti==1){
      if((actualPosition + 1) == amountOfImages){
        actualPosition = 0;
        temp=files.at(0);
      }
      else{
        temp=files.at(actualPosition + 1);
        actualPosition += 1;
      }
    }
    if(gti==-1){
      if((actualPosition) == 0){
        temp=files.at(amountOfImages - 1);
        actualPosition = amountOfImages - 1;
      }
      else{
        temp=files.at(actualPosition - 1);
        actualPosition -= 1;
      }
    }
    if(gti==2){
      if((actualPosition + 1) == amountOfImages){
        return false;
      }
      else{
        temp=files.at(amountOfImages - 1);
        actualPosition = amountOfImages - 1;
      }
    }
    if(gti==-2){
      if(actualPosition == 0){
        return false;
      }
      else{
        temp=files.at(0);
        actualPosition = 0;
      }
    }
    temp.prepend(archivo->absolutePath() + "/");
    archivo->setFile(temp);
    return true;
}

void MainWindow::goFirst()
{
  if(goToImage(-2))
    openImage();
}

void MainWindow::goLast()
{
  if(goToImage(2))
    openImage();
}

void MainWindow::next()
{
  if(goToImage(1))
    openImage();
}

void MainWindow::previous()
{
  if(goToImage(-1))
    openImage();
}

void MainWindow::openImage()
{
    determinateImageType();

    this->setCursor(Qt::BusyCursor);
    switch(imageType){
    case Static:
        openStatic();
        break;
    case Dynamic:
        openDynamic();
        break;
    case Webp:
        openStatic();
    }
    this->setCursor(Qt::ArrowCursor);
    pushNewFile();
    this->updateWindowTitle();
}

void MainWindow::determinateImageType()
{
#ifdef WEBP_SUPPORT
    if(archivo->absoluteFilePath().indexOf(".webp") != -1){
        imageType = Webp;
        return;
    }
#endif

    QMovie m(archivo->absoluteFilePath());
    if(m.isValid()){
        if(m.frameCount() > 1){
            imageType = Dynamic;
        }
        else{
            imageType = Static;
        }
    }
}

void MainWindow::updateWindowTitle()
{
    QString windowTitleFormat = QString("appName :: pos/amount - zoomLevel - imageName");
    windowTitleFormat.replace("appName", "QIviewer");
    windowTitleFormat.replace("imageName", archivo->fileName());
    windowTitleFormat.replace("pos", QString("%1").arg(actualPosition + 1));
    windowTitleFormat.replace("amount", QString("%1").arg(files.size()));
    windowTitleFormat.replace("zoomLevel", QString("%1%").arg(zoomWidget->getZoomValue()));
    this->setWindowTitle(windowTitleFormat);
}

void MainWindow::openStatic()
{
    imageWidget->setPicture(archivo->absoluteFilePath());
    zoomWidget->clearSizes();
    zoomWidget->setPicSize(imageWidget->getPictureSize());
    zoomWidget->setMaxSize(centralWidget()->size() - QSize(20,20));
    if(showZoomSlider){
        zoomSlider->setValue(zoomWidget->getZoomValue());
    }
    updateActions();
}

void MainWindow::openDynamic()
{
    imageWidget->setMovie(archivo->absoluteFilePath());
    zoomWidget->clearSizes();
    zoomWidget->setPicSize(imageWidget->getPictureSize());
    zoomWidget->setMaxSize(centralWidget()->size() - QSize(20,20));
    if(showZoomSlider){
        zoomSlider->setValue(zoomWidget->getZoomValue());
    }
    updateActions();
}

void MainWindow::couldOpen(bool d)
{
    if(d){
        imageSetted = true;
        return;
    }
    int h = QMessageBox::information(this, tr("QIviewer"), tr("I couldn't make it :'("),
                                     QMessageBox::Ok, QMessageBox::Open, QMessageBox::Ignore);
    switch(h){
    case QMessageBox::Ok:
        break;
    case QMessageBox::Open:
        this->open();
        break;
    case QMessageBox::Ignore:
        archivo->setFile(imageWidget->getPrevFile());
        exploreFolder();
        openImage();
        break;
    }
}

void MainWindow::wheelZoom(int d)
{
    if(!imageSetted)
        return;

    if(d > 0)
        zoomWidget->zoomIn();

    if(d < 0)
        zoomWidget->zoomOut();
    
    adjustSizeAct->setChecked(zoomWidget->bestZoomValue());
}

void MainWindow::zoomOut()
{
    zoomWidget->zoomOut();
    adjustSizeAct->setChecked(zoomWidget->bestZoomValue());
}

void MainWindow::adjustSizeSlot()
{
    zoomWidget->setAdjustedSize(adjustSizeAct->isChecked());
}

void MainWindow::zoomIn()
{
    zoomWidget->zoomIn();
    adjustSizeAct->setChecked(zoomWidget->bestZoomValue());
}

void MainWindow::normalSize()
{
    zoomWidget->normalSize();
    adjustSizeAct->setChecked(zoomWidget->bestZoomValue());
    zoomWidget->setAdjustedSize(false);
}

void MainWindow::getColorFromSettings(QStringList d)
{
    imageWidget->setSCBColor(QColor(d.at(0).toInt(),
                             d.at(1).toInt(),
                             d.at(2).toInt()));
}

void MainWindow::setShortcuts(bool n, QStringList d1, QStringList d2)
{
    if(n){
      openAct->setShortcut(tr("Ctrl+O"));
      openDirAct->setShortcut(tr("Ctrl+Shift+O"));
      saveAct->setShortcut(tr("Ctrl+S"));
      zoomInAct->setShortcut(tr("+"));
      exitAct->setShortcut(tr("Ctrl+Q"));
      zoomOutAct->setShortcut(tr("-"));
      normalSizeAct->setShortcut(tr("1"));
      adjustSizeAct->setShortcut(tr("0"));
      configAct->setShortcut(tr("Ctrl+C"));
      filePropertiesAct->setShortcut(tr("Ctrl+P"));
      aboutAct->setShortcut(tr("Ctrl+H"));
      rotateRightAct->setShortcut(tr("Ctrl+Shift+Right"));
      rotateLeftAct->setShortcut(tr("Ctrl+Shift+Left"));
      showMenuBarAct->setShortcut(tr("Ctrl+M"));
      nextAct->setShortcut(Qt::Key_Right);
      goFirstAct->setShortcut(Qt::Key_Home);
      prevAct->setShortcut(Qt::Key_Left);
      goLastAct->setShortcut(Qt::Key_End);
    }

    else{
        QString g;
        for(int i=0; i<d1.size(); i++){
            for(int j=0; j<actionsList.size(); j++){
                g = actionsList.at(j)->text();
                g.remove('&');
                if(g == d1.at(i)){
                    actionsList.at(j)->setShortcut(QKeySequence(d2.at(i)));
                }
            }
        }
    }
}

void MainWindow::loadSettings()
{
    settings->loadSettings();

    /*main app settings*/
    resize(settings->getWindowSize());
    getColorFromSettings(settings->getBackgroundColor());
    showMenuBarAct->setChecked(settings->getShowMenuBar());
    adjustSizeAct->setChecked(settings->getLoadFixedSize());
    zoomWidget->setZoomIncrement(settings->getZoomIncrement());
    zoomWidget->setAdjustedSize(settings->getLoadFixedSize());
    imageWidget->setBGType(settings->getBGToShow());
    imageWidget->setCBSSize(settings->getSquaresSize());
    imageWidget->setRestartWhenZooming(settings->getRestartWhenZooming());
    imageWidget->setMovieSpeed(settings->getMovieSpeed());
    imageWidget->setStopWhenFinish(settings->getStopMovieWhenFinish());
    imageWidget->invertColors(settings->imageJokes());
    showZoomSlider  = settings->getShowZoomSlider();
    compressLevel   = settings->getCompressLevel();
    maxRecentFiles  = settings->getMaxRecentFiles();
    recentFilesPath = settings->getRecentFilesList();
    this->createRFActs();
    this->adjustSizeSlot();
    this->showMenuBar();

    //tool bar settings
    mainToolBar->setMovable(settings->getTBMovable());
    setTBMovableAct->setChecked(!settings->getTBMovable());
    configureToolBar(settings->getActionsLoaded(),
                     Qt::ToolBarArea(settings->getTBArea()),
                     Qt::ToolButtonStyle(settings->getTBButtomStyle()),
                     settings->getTBVisible(),
                     showZoomSlider);

    //last dir used
    lastDirUsed = settings->getLastDirUsed();
    defaultPath = settings->getDefaultPath();
    pathToUse   = settings->getPathToUse();

    //shortcuts
    this->setShortcuts(true, //settings->getCreateNewShortCuts(),
                       QStringList(),//settings->getShortCutsActs(),
                       QStringList());//settings->getShortCuts());
    this->updateActions();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
  saveSettings();
  event->accept();
}

void MainWindow::saveSettings()
{
    //much more clear than loadSettings(), don't you think?
    settings->setLoadFixedSize(adjustSizeAct->isChecked());
    settings->setWindowSize(this->size());
    settings->setRecentFilesList(recentFilesPath);
    settings->setShowMenuBar(showMenuBarAct->isChecked());
    settings->setTBMovable(mainToolBar->isMovable());
    settings->setTBArea(int(this->toolBarArea(mainToolBar)));
    settings->setTBVisible(mainToolBar->isVisible());//necesary?
    settings->setLastDirUsed(lastDirUsed);
    /*settings->setCreateNewShortCuts(false);
    QStringList a, b;
    for(int i=0; i<actionsList.size(); i++){
        if(!actionsList.at(i)->shortcut().isEmpty()){
            a<<actionsList.at(i)->text().remove('&');
            b<<actionsList.at(i)->shortcut();
        }
    }
    settings->setShortCuts(a, b);*/
    settings->saveSettings();
}
